@@ -243,19 +243,16 @@ typedef unsigned sasl_ssf_t;
  *                         dictionary attack
  * FORWARD_SECRECY      -- require forward secrecy between sessions
  *                         (breaking one won't help break next)
- * MAXIMUM              -- require all security attributes
- *		           known by the library at runtime
- * PASS_CREDENTIALS     -- prefer mechanisms which pass client
- *			   credentials (if available),
- *			   and allow mechanisms which can pass
+ * NOANONYMOUS          -- don't permit mechanisms that allow anonymous login
+ * PASS_CREDENTIALS     -- require mechanisms which pass client
+ *			   credentials, and allow mechanisms which can pass
  *			   credentials to do so
  */
 #define SASL_SEC_NOPLAINTEXT     (0x0001)
 #define SASL_SEC_NOACTIVE        (0x0002)
 #define SASL_SEC_NODICTIONARY    (0x0004)
 #define SASL_SEC_FORWARD_SECRECY (0x0008)
-#define SASL_SEC_MAX_DEFINED     (0x000F)
-#define SASL_SEC_MAXIMUM         (0x0100)
+#define SASL_SEC_NOANONYMOUS     (0x0010)
 #define SASL_SEC_PASS_CREDENTIALS (0x0200)
 
 typedef struct sasl_security_properties 
@@ -384,6 +381,24 @@ typedef int sasl_getpath_t(void * context,
 
 #define SASL_CB_GETPATH	    (3)
 
+/* verify file callback -- this allows applications to check if they
+ * want SASL to use files, file by file.  This is intended to allow
+ * applications to sanity check the environment to make sure plugins
+ * or the configuration file can't be written to, etc.
+ * inputs: 
+ *  context     -- verifypath context from the callback record
+ *  file        -- full path to file to verify
+
+ * returns:
+ *  SASL_OK        -- no error (file can safely be used)
+ *  SASL_CONTINUE  -- continue WITHOUT using this file
+ *  SASL_FAIL      -- error 
+ */
+typedef int sasl_verifyfile_t(void * context,
+                              const char * file);
+
+#define SASL_CB_VERIFYFILE  (4)
+
 /* client/user interaction callbacks:
  */
 /* Simple prompt -- result must persist until next call to getsimple or
@@ -542,6 +557,7 @@ LIBSASL_API int sasl_setprop(sasl_conn_t *conn,
 #define SASL_SSF_EXTERNAL 100  /* external SSF active --
 				* sasl_external_properties_t */
 #define SASL_SEC_PROPS    101  /* sasl_security_properties_t */
+			       /* also allows SASL_IP_LOCAL, SASL_IP_REMOTE */
 
 /* do precalculations during an idle period or network round trip
  *  may pass NULL to precompute for some mechanisms prior to connect
@@ -692,9 +708,9 @@ LIBSASL_API int sasl_server_init(const sasl_callback_t *callbacks,
 
 /* create context for a single SASL connection
  *  service        -- registered name of the service using SASL (e.g. "imap")
- *  local_domain   -- Fully qualified local domain name.  May be NULL
- *                    for default domain.  Useful for multi-homed servers.
- *  user_domain    -- permits multiple user domains on server, NULL = default
+ *  serverFQDN     -- Fully qualified server domain name.  NULL means use
+ *                    gethostbyname().  Useful for multi-homed servers.
+ *  user_realm     -- permits multiple user domains on server, NULL = default
  *  callbacks      -- callbacks (e.g., authorization, lang, new getopt context)
  *  secflags       -- security flags (see above)
  * returns:
@@ -705,8 +721,8 @@ LIBSASL_API int sasl_server_init(const sasl_callback_t *callbacks,
  *  SASL_NOMEM     -- not enough memory
  */
 LIBSASL_API int sasl_server_new(const char *service,
-		    const char *local_domain,
-		    const char *user_domain,
+		    const char *serverFQDN,
+		    const char *user_realm,
 		    const sasl_callback_t *callbacks,
 		    int secflags,
 		    sasl_conn_t **pconn);
@@ -783,7 +799,7 @@ LIBSASL_API int sasl_server_step(sasl_conn_t *conn,
 /* check if a plaintext password is valid
  * if user is NULL, check if plaintext is enabled
  * inputs:
- *  user         -- user to query in current user_domain
+ *  user         -- user to query in current user_realm
  *  userlen      -- length of username, 0 = strlen(user)
  *  pass         -- plaintext password to check
  *  passlen      -- length of password, 0 = strlen(pass)
@@ -803,7 +819,7 @@ LIBSASL_API int sasl_checkpass(sasl_conn_t *conn,
 
 /* check if a user exists on server
  *  service       -- registered name of the service using SASL (e.g. "imap")
- *  user_domain   -- permits multiple user domains on server, NULL = default
+ *  user_realm    -- permits multiple user domains on server, NULL = default
  *  user          -- NUL terminated user name
  *
  * returns:
@@ -813,7 +829,7 @@ LIBSASL_API int sasl_checkpass(sasl_conn_t *conn,
  *  SASL_NOMECH   -- user found, but no usable mechanism
  */
 LIBSASL_API int sasl_user_exists(const char *service,
-		     const char *user_domain,
+		     const char *user_realm,
 		     const char *user);
 
 /* set the password for a user
