@@ -92,6 +92,10 @@ typedef struct sasl_utils {
     void (*rand)(sasl_rand_t *rpool, char *buf, unsigned len);
     void (*churn)(sasl_rand_t *rpool, const char *data, unsigned len);
 
+    /* current CMU hack.  DO NOT USE EXCEPT IN PLAIN */
+    int (*checkpass)(const char *mech, const char *service,
+		     const char *user, const char *pass);
+
     /* callback function */
     sasl_getcallback_t *getcallback;
 
@@ -186,7 +190,7 @@ typedef struct sasl_client_params {
     const char *service;	  /* service name */
     const char *pass;		  /* plaintext passphrase, if used */
     const char *serverFQDN;	  /* server fully qualified domain name */
-    const char *local_domain;	  /* client's local domain name */
+    const char *clientFQDN;	  /* client's local domain name */
     sasl_utils_t *utils;	  /* SASL API utility routines */
     sasl_mech_secret_t *secret;	  /* mech-specific decrypted secret */
 
@@ -324,8 +328,8 @@ typedef struct sasl_server_params {
     const char *service;	/* NULL = default service for user_exists
 				   and setpass */
     const char *appname;	/* name of calling application */
-    const char *local_domain;	/* local domain name */
-    const char *user_domain;	/* set of users who are active */
+    const char *serverFQDN;	/* local domain name */
+    const char *user_realm;	/* set of users who are active */
     sasl_utils_t *utils;	/* SASL API utility routines */
 
     /* application's security requirements */
