@@ -126,7 +126,7 @@ int sasl_decode(sasl_conn_t *conn, const char *input, unsigned inputlen,
 {
   if (! conn || ! input || ! output || ! outputlen)
     return SASL_FAIL;
-  if (conn->oparams.encode==NULL)
+  if (conn->oparams.decode==NULL)
   {
     /* just copy the string, no encryption */
     *output = sasl_ALLOC(inputlen + 1);
@@ -169,7 +169,7 @@ int _sasl_conn_init(sasl_conn_t *conn,
 		    const char *service,
 		    int secflags,
 		    int (*idle_hook)(sasl_conn_t *conn),
-		    const char *local_domain,
+		    const char *serverFQDN,
 		    const sasl_callback_t *callbacks,
 		    const sasl_global_callbacks_t *global_callbacks) {
   int result = SASL_OK;
@@ -198,25 +198,16 @@ int _sasl_conn_init(sasl_conn_t *conn,
   conn->callbacks = callbacks;
   conn->global_callbacks = global_callbacks;
 
-  if (local_domain==NULL) {
+  if (serverFQDN==NULL) {
     char name[MAXHOSTNAMELEN];
     memset(name, 0, sizeof(name));
     gethostname(name, MAXHOSTNAMELEN);
-#ifdef HAVE_GETDOMAINNAME
-    {
-      char *dot = strchr(name, '.');
-      if (! dot) {
-	size_t namelen = strlen(name);
-	name[namelen] = '.';
-	getdomainname(name + namelen + 1, MAXHOSTNAMELEN - namelen - 1);
-      }
-    }
-#endif /* HAVE_GETDOMAINNAME */
-    result = _sasl_strdup(name, &conn->local_domain, NULL);
+
+    result = _sasl_strdup(name, &conn->serverFQDN, NULL);
     if (result != SASL_OK)
       goto cleanup_mutex;
   } else {
-    result = _sasl_strdup(local_domain, &conn->local_domain, NULL);
+    result = _sasl_strdup(serverFQDN, &conn->serverFQDN, NULL);
     if (result != SASL_OK)
       goto cleanup_mutex;
   }
@@ -259,11 +250,19 @@ void _sasl_conn_dispose(sasl_conn_t *conn) {
 
   sasl_MUTEX_DISPOSE(conn->mutex);
 
-  if (conn->local_domain)
-    sasl_FREE(conn->local_domain);
+  if (conn->serverFQDN)
+    sasl_FREE(conn->serverFQDN);
 }
 
 
+/* get property from SASL connection state
+ *  propnum       -- property number
+ *  pvalue        -- pointer to value
+ * returns:
+ *  SASL_OK       -- no error
+ *  SASL_NOTDONE  -- property not available yet
+ *  SASL_BADPARAM -- bad property number
+ */
 int sasl_getprop(sasl_conn_t *conn, int propnum, void **pvalue)
 {
   int result;
@@ -280,34 +279,35 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, void **pvalue)
       if (! conn->oparams.user)
 	result = SASL_NOTDONE;
       else
-	*pvalue=conn->oparams.user;
+	*(char **)pvalue=conn->oparams.user;
       break;
     case SASL_SSF:
-      *(sasl_ssf_t *)pvalue= conn->oparams.mech_ssf;
+      *(sasl_ssf_t **)pvalue= &conn->oparams.mech_ssf;
       break;      
     case SASL_MAXOUTBUF:
-      *(unsigned *)pvalue = conn->oparams.maxoutbuf;
+      *(unsigned **)pvalue = &conn->oparams.maxoutbuf;
       break;
     case SASL_REALM:
       if (! conn->oparams.realm)
 	result = SASL_NOTDONE;
       else
-	*pvalue = conn->oparams.realm;
+	*(char **)pvalue = conn->oparams.realm;
       break;
     case SASL_GETOPTCTX:
+      result = SASL_FAIL;
       /* ??? */
       break;
     case SASL_IP_LOCAL:
       if (! conn->got_ip_local)
 	result = SASL_NOTDONE;
       else
-	*(struct sockaddr_in *)pvalue = conn->ip_local;
+	*(struct sockaddr_in **)pvalue = &conn->ip_local;
       break;
     case SASL_IP_REMOTE:
       if (! conn->got_ip_remote)
 	result = SASL_NOTDONE;
       else
-	*(struct sockaddr_in *)pvalue = conn->ip_remote;
+	*(struct sockaddr_in **)pvalue = &conn->ip_remote;
       break;
     default: 
       result = SASL_BADPARAM;
@@ -316,6 +316,11 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, void **pvalue)
   return result; 
 }
 
+/* set property in SASL connection state
+ * returns:
+ *  SASL_OK       -- value set
+ *  SASL_BADPARAM -- invalid property or value
+ */
 int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 {
   int result;
@@ -345,7 +350,7 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	str = NULL;
       if (conn->external.auth_id)
 	sasl_FREE(conn->external.auth_id);
-      conn->external.auth_id = NULL;
+      conn->external.auth_id = str;
       conn->external.ssf = external->ssf;
       break;
     }
@@ -447,9 +452,13 @@ _sasl_global_getopt(void *context,
 	      == SASL_OK))
 	return SASL_OK;
 
-  /* TODO: Someday, we ought to look up options in a global
-   * configuration file of some sort.  For now, though, we
-   * just default to pretending the option doesn't exist. */
+  /* look it up in our configuration file */
+  *result = sasl_config_getstring(option, NULL);
+  if (*result != NULL) {
+      if (len) { *len = strlen(*result); }
+      return SASL_OK;
+  }
+
   return SASL_FAIL;
 }
 
@@ -565,6 +574,14 @@ _sasl_getpath(void *context __attribute__((unused)),
   return _sasl_strdup(path, path_dest, NULL);
 }
 
+static int
+_sasl_verifyfile(void *context __attribute__((unused)),
+		 char *file  __attribute__((unused)))
+{
+  /* always say ok */
+  return SASL_OK;
+}
+
 int
 _sasl_getcallback(sasl_conn_t * conn,
 		  unsigned long callbackid,
@@ -641,6 +658,10 @@ _sasl_getcallback(sasl_conn_t * conn,
     *pproc = _sasl_server_putsecret_hook;
     *pcontext = NULL;
     return SASL_OK;
+  case SASL_CB_VERIFYFILE:
+    *pproc = & _sasl_verifyfile;
+    *pcontext = NULL;
+    return SASL_OK;
   }
 
   /* Unable to find a callback... */
@@ -782,25 +803,27 @@ _sasl_log (sasl_conn_t *conn,
 	    done=1;
 	    break;
 
-	  /* only d, i, o, u, x, or X 
-	     c, e,  E, f,  g, or G can end the format thing */
 	  case 'c':
+	    frmt[frmtpos++]=format[pos];
+	    frmt[frmtpos]=0;
+	    tempbuf[0] = va_arg(ap, char); /* get the next arg */
+	    tempbuf[1]='\0';
+	    
+	    /* now add the character */
+	    result = add_string(&out, &alloclen, &outlen, tempbuf);
+	    if (result != SASL_OK)
+	      return result;
+	    done=1;
+
+	    break;
+
 	  case 'd':
 	  case 'i':
-	  case 'o':
-	  case 'u':
-	  case 'x':
-	  case 'X':
-	  case 'e':
-	  case 'E':
-	  case 'f':
-	  case 'g':
-	  case 'G':
 	    frmt[frmtpos++]=format[pos];
 	    frmt[frmtpos]=0;
 	    ival = va_arg(ap, int); /* get the next arg */
 
-	    snprintf(tempbuf,20,frmt,ival); /* have sprintf do the work */
+	    snprintf(tempbuf,20,frmt,ival); /* have snprintf do the work */
 	    /* now add the string */
 	    result = add_string(&out, &alloclen, &outlen, tempbuf);
 	    if (result != SASL_OK)
@@ -878,7 +901,8 @@ _sasl_alloc_utils(sasl_conn_t *conn,
   utils->mkchal = &sasl_mkchal;
   utils->utf8verify = &sasl_utf8verify;
   utils->rand=&sasl_rand;
-  utils->churn=&sasl_churn;
+  utils->churn=&sasl_churn;  
+  utils->checkpass=NULL;
   
   utils->getcallback=&_sasl_getcallback;
 
@@ -939,3 +963,26 @@ _sasl_find_getpath_callback(const sasl_callback_t *callbacks)
   return &default_getpath_cb;
 }
 
+const sasl_callback_t *
+_sasl_find_verifyfile_callback(const sasl_callback_t *callbacks)
+{
+  static const sasl_callback_t default_verifyfile_cb = {
+    SASL_CB_VERIFYFILE,
+    &_sasl_verifyfile,
+    NULL
+  };
+
+  if (callbacks)
+    while (callbacks->id != SASL_CB_LIST_END)
+    {
+      if (callbacks->id == SASL_CB_VERIFYFILE)
+      {
+	return callbacks;
+      } else {
+	++callbacks;
+      }
+    }
+  
+  return &default_verifyfile_cb;
+}
+
