@@ -35,13 +35,28 @@ SOFTWARE.
 # ifdef _POSIX_PATH_MAX
 #  define PATH_MAX _POSIX_PATH_MAX
 # else
-#  define PATH_MAX 1024		/* arbitrary; probably big enough */
+#  define PATH_MAX 1024		/* arbitrary; probably big enough will
+				 * probably only be 256+64 on
+				 * pre-posix machines */
 # endif
 #endif
 
+#ifndef NAME_MAX
+# ifdef _POSIX_NAME_MAX
+#  define NAME_MAX _POSIX_NAME_MAX
+# else
+#  define NAME_MAX 16
+# endif
+#endif
+ 
+#if NAME_MAX < 8
+#  define NAME_MAX 8
+#endif
+
 /* gets the list of mechanisms */
 int _sasl_get_mech_list(const char *entryname,
 			const sasl_callback_t *getpath_cb,
+			const sasl_callback_t *verifyfile_cb,
 			int (*add_plugin)(void *,void *))
 {
   /* XXX These fixed-length buffers could be a problem;
@@ -60,12 +75,15 @@ int _sasl_get_mech_list(const char *entryname,
       || ! getpath_cb
       || getpath_cb->id != SASL_CB_GETPATH
       || ! getpath_cb->proc
+      || ! verifyfile_cb
+      || verifyfile_cb->id != SASL_CB_VERIFYFILE
+      || ! verifyfile_cb->proc
       || ! add_plugin)
     return SASL_BADPARAM;
 
+  /* get the path to the plugins */
   result = ((sasl_getpath_t *)(getpath_cb->proc))(getpath_cb->context,
 						  &path);
-
   if (result != SASL_OK)
     return result;
 
@@ -74,7 +92,7 @@ int _sasl_get_mech_list(const char *entryname,
     path = PLUGINDIR;
   } else
     free_path = 1;
-
+  /* xxx sendmail guys -NAME_MAX */
   if (strlen(path)>=PATH_MAX) { /* no you can't buffer overrun */
     if (free_path)
       sasl_FREE(path);
@@ -90,14 +108,13 @@ int _sasl_get_mech_list(const char *entryname,
       str[pos]=c;
       pos++;
     } while ((c!=':') && (c!='=') && (c!=0));
-    str[pos-1]=0;
+    str[pos-1]='\0';
 
     strcpy(prefix,str);
     strcat(prefix,"/");
-    
+
     if ((dp=opendir(str)) !=NULL) /* ignore errors */    
     {
-      strcat(str, "/");
       while ((dir=readdir(dp)) != NULL)
       {
 	size_t length;
@@ -105,20 +122,31 @@ int _sasl_get_mech_list(const char *entryname,
 	void *entry_point;
 
 	length = NAMLEN(dir);
-	if (length < 4) continue;
+	if (length < 4) continue; /* can not possibly be what we're looking for */
+
+	if (length + pos>=PATH_MAX) continue; /* too big */
+
 	if (strcmp(dir->d_name + (length - 3), ".so")) continue;
 	{
 	  char name[PATH_MAX];
 	  memcpy(name,dir->d_name,length);
-	  name[length]=0;
+	  name[length]='\0';
 
 	  strcpy(tmp,prefix);
 	  strcat(tmp,name);
 
 	  VL(("entry is = [%s]\n",tmp));
 
+	  /* Ask the application if we should use this file or not */
+	  result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
+								tmp);	  
+	  /* returns continue if this file is to be skipped */
+	  if (result==SASL_CONTINUE) continue; 
+	  
+	  if (result!=SASL_OK) return result;
+	  
 	  library=NULL;
-	  if (!(library=dlopen(tmp,RTLD_LAZY)))
+	  if (!(library=dlopen(tmp,RTLD_NOW))) /* xxx no RTLD_LOCAL | on linux */
 	    {
 	      VL(("Unable to dlopen %s: %s\n", tmp, dlerror()));
 	      continue;
@@ -139,7 +167,7 @@ int _sasl_get_mech_list(const char *entryname,
 	  dlclose(library);
 	  continue;
 	}
-	VL(("added [%s] sucessfully\n",dir->d_name));
+	VL(("added [%s] successfully\n",dir->d_name));
       }
 
      closedir(dp);
