@@ -32,11 +32,15 @@ SOFTWARE.
 
 extern int _sasl_get_mech_list(const char *entryname,
 			       const sasl_callback_t *getpath_callback,
+			       const sasl_callback_t *verifyfile_callback,
 			       int (*add_plugin)(void *,void *));
 
 extern const sasl_callback_t *
 _sasl_find_getpath_callback(const sasl_callback_t *callbacks);
 
+extern const sasl_callback_t *
+_sasl_find_verifyfile_callback(const sasl_callback_t *callbacks);
+
 extern int _sasl_done_with_plugin(void *plugin);
 
 extern void (*_sasl_client_cleanup_hook)(void);
@@ -82,7 +86,7 @@ struct sasl_conn {
   const sasl_global_callbacks_t *global_callbacks; /* global callbacks
 						    * for this
 						    * connection */
-  char *local_domain;
+  char *serverFQDN;
 };
 
 extern int _sasl_conn_init(sasl_conn_t *conn,
@@ -156,4 +160,19 @@ _sasl_log(sasl_conn_t *conn,
 	  const char *format,
 	  ...);
 
+/* config file declarations (config.c) */
+extern int sasl_config_init(const char *filename);
+extern const char *sasl_config_getstring(const char *key,const char *def);
+extern int sasl_config_getint(const char *key,int def);
+extern int sasl_config_getswitch(const char *key,int def);
+
+/* clear password checking declarations (checkpw.c) */
+extern int _sasl_passwd_verify_password(const char *userid,const char *password, const char **reply);
+extern int _sasl_shadow_verify_password(const char *userid,const char *password, const char **reply);
+extern int _sasl_kerberos_verify_password(const char *user,
+					  const char *passwd,
+					  const char *service,
+					  const char **reply);
+extern int _sasl_PAM_verify_password(const char *userid,const char *password, const char **reply);
+
 #endif /* SASLINT_H */
