@@ -34,15 +34,49 @@ SOFTWARE.
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
+
+#ifdef HAVE_PAM
+#define DEFAULT_PLAIN_MECHANISM "PAM"
+#else
+# ifdef HAVE_KRB
+#define DEFAULT_PLAIN_MECHANISM "kerberos_v4"
+# else
+#define DEFAULT_PLAIN_MECHANISM "passwd"
+# endif
+#endif
+
 #include "sasl.h"
 #include "saslint.h"
 #include "saslutil.h"
 
+
 #ifdef sun
 /* gotta define gethostname ourselves on suns */
 extern int gethostname(char *, int);
 #endif
 
+#ifndef PATH_MAX
+# ifdef _POSIX_PATH_MAX
+#  define PATH_MAX _POSIX_PATH_MAX
+# else
+#  define PATH_MAX 1024		/* arbitrary; probably big enough */
+# endif
+#endif
+
+/* Contains functions:
+ * 
+ * sasl_server_init
+ * sasl_server_new
+ * sasl_listmech
+ * sasl_server_start
+ * sasl_server_step
+ * sasl_checkpass
+ * sasl_userexists <= not yet implemented
+ * sasl_setpass
+ */
+
+static int _sasl_checkpass(const char *mech, const char *service, 
+			   const char *user, const char *pass);
 
 static int
 external_server_new(void *glob_context __attribute__((unused)),
@@ -199,7 +233,8 @@ typedef struct mechanism
   int version;
   const sasl_server_plug_t *plug;
   struct mechanism *next;
-  void *library;
+  void *library; /* this a pointer to shared library returned by dlopen 
+		    or some similar function on other platforms */
 } mechanism_t;
 
 
@@ -215,7 +250,8 @@ typedef struct mech_list {
 typedef struct sasl_server_conn {
   sasl_conn_t base; /* parts common to server + client */
 
-  char *user_domain;
+  char *user_realm; /* domain the user authenticating is in This is
+		      * usually simply their hostname */
 
   int authenticated;
   mechanism_t *mech; /* mechanism trying to use */
@@ -237,16 +273,23 @@ static mech_list_t *mechlist; /* global var which holds the list */
 
 static sasl_global_callbacks_t global_callbacks;
 
-/* Contains functions:
+
+/* set the password for a user
+ *  conn        -- SASL connection
+ *  user        -- user name
+ *  pass        -- plaintext password, may be NULL to remove user
+ *  passlen     -- length of password, 0 = strlen(pass)
+ *  flags       -- see flags below
+ *  errstr      -- optional more detailed error
  * 
- * sasl_server_init
- * sasl_server_new
- * sasl_listmech
- * sasl_server_start
- * sasl_server_step
- * sasl_checkpass NTI
- * sasl_userexists NTI
- * sasl_setpass
+ * returns:
+ *  SASL_NOCHANGE  -- proper entry already exists
+ *  SASL_NOMECH    -- no authdb supports password setting as configured
+ *  SASL_DISABLED  -- account disabled
+ *  SASL_PWLOCK    -- password locked
+ *  SASL_FAIL      -- OS error
+ *  SASL_BADPARAM  -- password too long
+ *  SASL_OK        -- successful
  */
 
 int sasl_setpass(sasl_conn_t *conn,
@@ -274,9 +317,9 @@ int sasl_setpass(sasl_conn_t *conn,
       user, passlen, pass, passlen));
 
   /* copy info into sparams */
-  s_conn->sparams->local_domain=conn->local_domain;
+  s_conn->sparams->serverFQDN=conn->serverFQDN;
   s_conn->sparams->service=conn->service;
-  s_conn->sparams->user_domain=s_conn->user_domain;
+  s_conn->sparams->user_realm=s_conn->user_realm;
 
   _sasl_log(conn,
 	    SASL_LOG_WARNING,
@@ -343,8 +386,8 @@ static void server_dispose(sasl_conn_t *pconn)
     s_conn->mech->plug->mech_dispose(pconn->context,
 				     s_conn->sparams->utils);
 
-  if (s_conn->user_domain)
-    sasl_FREE(s_conn->user_domain);
+  if (s_conn->user_realm)
+    sasl_FREE(s_conn->user_realm);
 
   _sasl_free_utils(&s_conn->sparams->utils);
 
@@ -358,13 +401,19 @@ static int init_mechlist(void)
 {
   /* set util functions - need to do rest*/
   mechlist->utils=_sasl_alloc_utils(NULL, &global_callbacks);
-
+  mechlist->utils->checkpass = &_sasl_checkpass;
+  
   if (mechlist->utils==NULL)
     return SASL_NOMEM;
 
   return SASL_OK;
 }
 
+/*
+ * parameters:
+ *  p - entry point
+ *  library - shared library ptr returned by dlopen
+ */
 static int add_plugin(void *p, void *library) {
   int plugcount;
   const sasl_server_plug_t *pluglist;
@@ -376,6 +425,8 @@ static int add_plugin(void *p, void *library) {
 
   entry_point = (sasl_server_plug_init_t *)p;
 
+  /* call into the shared library asking for information about it */
+  /* version is filled in with the version of the plugin */
   result = entry_point(mechlist->utils, SASL_SERVER_PLUG_VERSION, &version,
 		       &pluglist, &plugcount);
 
@@ -392,17 +443,24 @@ static int add_plugin(void *p, void *library) {
     result = SASL_FAIL;
   }
 
-  for (lupe=0;lupe< plugcount ;lupe++)
+  for (lupe=0;lupe < plugcount ;lupe++)
     {
       mech = sasl_ALLOC(sizeof(mechanism_t));
       if (! mech) return SASL_NOMEM;
 
       mech->plug=pluglist++;
       mech->version = version;
+
+      /*
+       * We want plugin library to close but we only
+       * want to do this once per plugin regardless
+       * of how many mechs are in a single plugin 
+       */
       if (lupe==0)
 	mech->library=library;
       else
 	mech->library=NULL;
+
       mech->next = mechlist->mech_list;
       mechlist->mech_list = mech;
 
@@ -440,7 +498,7 @@ server_idle(sasl_conn_t *conn)
     return 0;
 
   for (m = mechlist->mech_list;
-       m;
+       m!=NULL;
        m = m->next)
     if (m->plug->idle
 	&&  m->plug->idle(m->plug->glob_context,
@@ -450,6 +508,69 @@ server_idle(sasl_conn_t *conn)
   return 0;
 }
 
+static int load_config(const sasl_callback_t *verifyfile_cb)
+{
+  int result;
+  char *path_to_config=NULL;
+  char *config_filename=NULL;
+  int len;
+  const sasl_callback_t *getpath_cb=NULL;
+
+  /* get the path to the plugins; for now the config file will reside there */
+  getpath_cb=_sasl_find_getpath_callback( global_callbacks.callbacks );
+  if (getpath_cb==NULL) return SASL_BADPARAM;
+
+  /* getpath_cb->proc MUST be a sasl_getpath_t; if only c had a type
+     system */
+  result = ((sasl_getpath_t *)(getpath_cb->proc))(getpath_cb->context,
+						  &path_to_config);
+  if (result!=SASL_OK) return result;
+  if (path_to_config == NULL) path_to_config = "";
+
+  /* length = length of path + '/' + length of appname + ".conf" + 1
+     for '\0' */
+  len = strlen(path_to_config)+2+ strlen(global_callbacks.appname)+5+1;
+
+  if (len > PATH_MAX ) return SASL_FAIL;
+
+  /* construct the filename for the config file */
+  config_filename = sasl_ALLOC(len);
+  if (! config_filename) return SASL_NOMEM; 
+
+  strcpy(config_filename, path_to_config);
+  strcat(config_filename,"/");
+  strcat(config_filename, global_callbacks.appname);
+  strcat(config_filename, ".conf");
+
+
+  /* Ask the application if it's safe to use this file */
+  result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
+							config_filename);	  
+  /* returns continue if this file is to be skipped */
+  
+  /* returns SASL_CONTINUE if doesn't exist
+   * if doesn't exist we can continue using default behavior
+   */
+  if (result==SASL_OK)
+    result=sasl_config_init(config_filename);
+
+  sasl_FREE(config_filename);
+
+  return result;
+}
+
+/* initialize server drivers, done once per process
+ *  callbacks      -- base callbacks for all server connections
+ *  appname        -- name of calling application (for lower level logging)
+ * results:
+ *  state          -- server state
+ * returns:
+ *  SASL_OK        -- success
+ *  SASL_BADPARAM  -- error in config file
+ *  SASL_NOMEM     -- memory failure
+ *  SASL_BADVERS   -- Mechanism version mismatch
+ */
+
 int sasl_server_init(const sasl_callback_t *callbacks,
 		     const char *appname)
 {
@@ -467,6 +588,10 @@ int sasl_server_init(const sasl_callback_t *callbacks,
   mechlist=sasl_ALLOC(sizeof(mech_list_t));
   if (mechlist==NULL) return SASL_NOMEM;
 
+  /* load config file if applicable */
+  ret=load_config(_sasl_find_verifyfile_callback(callbacks));
+  if ((ret!=SASL_OK) && (ret!=SASL_CONTINUE)) return ret;
+
   /* load plugins */
   ret=init_mechlist();
   if (ret!=SASL_OK)
@@ -478,11 +603,21 @@ int sasl_server_init(const sasl_callback_t *callbacks,
 
   ret=_sasl_get_mech_list("sasl_server_plug_init",
 			  _sasl_find_getpath_callback(callbacks),
+			  _sasl_find_verifyfile_callback(callbacks),
 			  &add_plugin);
 
   return ret;
 }
 
+/*
+ * Once we have the users plaintext password we 
+ * may want to transition them. That is put entries
+ * for them in the passwd database for other
+ * stronger mechanism
+ *
+ * for example PLAIN -> CRAM-MD5
+ */
+
 static int
 _sasl_transition(sasl_conn_t * conn,
 		 const char * pass,
@@ -494,6 +629,9 @@ _sasl_transition(sasl_conn_t * conn,
   if (! conn->oparams.authid)
     return SASL_NOTDONE;
 
+  /* check if this is enabled: default to false */
+  if (sasl_config_getswitch("auto_transition", 0) == 0) return SASL_OK;
+
   return sasl_setpass(conn,
 		      conn->oparams.authid,
 		      pass,
@@ -502,9 +640,25 @@ _sasl_transition(sasl_conn_t * conn,
 		      NULL);
 }
 
+
+/* create context for a single SASL connection
+ *  service        -- registered name of the service using SASL (e.g. "imap")
+ *  serverFQDN     -- Fully qualified server domain name.  NULL means use
+ *                    gethostbyname().  Useful for multi-homed servers.
+ *  user_realm     -- permits multiple user domains on server, NULL = default
+ *  callbacks      -- callbacks (e.g., authorization, lang, new getopt context)
+ *  secflags       -- security flags (see above)
+ * returns:
+ *  pconn          -- new connection context
+ *
+ * returns:
+ *  SASL_OK        -- success
+ *  SASL_NOMEM     -- not enough memory
+ */
+
 int sasl_server_new(const char *service,
-		    const char *local_domain,
-		    const char *user_domain,
+		    const char *serverFQDN,
+		    const char *user_realm,
 		    const sasl_callback_t *callbacks,
 		    int secflags,
 		    sasl_conn_t **pconn)
@@ -521,7 +675,7 @@ int sasl_server_new(const char *service,
   (*pconn)->destroy_conn = &server_dispose;
   result = _sasl_conn_init(*pconn, service, secflags,
 			   &server_idle,
-			   local_domain,
+			   serverFQDN,
 			   callbacks, &global_callbacks);
   if (result != SASL_OK) return result;
 
@@ -537,28 +691,35 @@ int sasl_server_new(const char *service,
   serverconn->sparams->utils=_sasl_alloc_utils(*pconn, &global_callbacks);
   if (serverconn->sparams->utils==NULL)
     return SASL_NOMEM;
+  serverconn->sparams->utils->checkpass = &_sasl_checkpass;
 
   serverconn->sparams->transition = &_sasl_transition;
 
   serverconn->sparams->props = serverconn->base.props;
 
   /* set some variables */
-  if (user_domain==NULL)
-    serverconn->user_domain=NULL;
+  if (user_realm==NULL)
+    serverconn->user_realm=NULL;
   else {
-    result = _sasl_strdup(user_domain, &serverconn->user_domain, NULL);
-    if (result != SASL_OK) goto cleanup_conn;
+    result = _sasl_strdup(user_realm, &serverconn->user_realm, NULL);
+  }
+
+  if (result!=SASL_OK)
+  {
+    _sasl_conn_dispose(*pconn);
+    sasl_FREE(*pconn);
+    *pconn = NULL;
   }
 
   return result;
-
-cleanup_conn:
-  _sasl_conn_dispose(*pconn);
-  sasl_FREE(*pconn);
-  *pconn = NULL;
-  return result;
 }
 
+/*
+ * The rule is:
+ * IF mech strength + external strength < min ssf THEN FAIL
+ * We also have to look at the security properties and make sure
+ * that this mechanism has everything we want
+ */
 static int mech_permitted(sasl_conn_t *conn,
 			  const sasl_server_plug_t *plug)
 {
@@ -567,7 +728,7 @@ static int mech_permitted(sasl_conn_t *conn,
     return 0;
   if (plug == &external_server_mech) {
     /* Special case for the external mechanism */
-    if (conn->props.min_ssf < conn->external.ssf
+    if (conn->props.min_ssf > conn->external.ssf
 	|| ! conn->external.auth_id)
       return 0;
   } else {
@@ -575,9 +736,32 @@ static int mech_permitted(sasl_conn_t *conn,
     if (plug->max_ssf < conn->props.min_ssf)
       return 0;
   }
+
+  /* security properties---if there are any flags that differ and are
+     in what the connection are requesting, then fail */
+
+  /* do we want to special case SASL_SEC_PASS_CREDENTIALS? nah.. */
+  if (((conn->props.security_flags ^ plug->security_flags) 
+       & conn->props.security_flags) != 0) {
+      return 0;
+  }
+
   return 1;
 }
 
+/* start a mechanism exchange within a connection context
+ *  mech           -- the mechanism name client requested
+ *  clientin       -- client initial response, NULL if empty
+ *  clientinlen    -- length of initial response
+ *  serverout      -- initial server challenge, NULL if done
+ *  serveroutlen   -- length of initial server challenge
+ * output:
+ *  pconn          -- the connection negotiation state on success
+ *  errstr         -- set to string to send to user on failure
+ *
+ * Same returns as sasl_server_step()
+ */
+
 int sasl_server_start(sasl_conn_t *conn,
 		      const char *mech,
 		      const char *clientin,
@@ -594,6 +778,11 @@ int sasl_server_start(sasl_conn_t *conn,
   mechanism_t *m;
   m=mechlist->mech_list;
 
+  /* check parameters */
+  if ((mech==NULL)    ||
+      ((clientin==NULL) && (clientinlen>0)))
+    return SASL_BADPARAM;
+
   if (errstr)
     *errstr = NULL;
 
@@ -616,9 +805,10 @@ int sasl_server_start(sasl_conn_t *conn,
   s_conn->mech=m;
 
   /* call the security layer given by mech */
-  s_conn->sparams->local_domain=conn->local_domain;
+  s_conn->sparams->serverFQDN=conn->serverFQDN;
   s_conn->sparams->service=conn->service;
-  s_conn->sparams->user_domain=s_conn->user_domain;
+  s_conn->sparams->user_realm=s_conn->user_realm;
+  s_conn->sparams->props=conn->props;
 
   result = s_conn->mech->plug->mech_new(s_conn->mech->plug->glob_context,
 					s_conn->sparams,
@@ -639,6 +829,25 @@ int sasl_server_start(sasl_conn_t *conn,
 				       errstr);
 }
 
+
+/* perform one step of the SASL exchange
+ *  inputlen & input -- client data
+ *                      NULL on first step if no optional client step
+ *  outputlen & output -- set to the server data to transmit
+ *                        to the client in the next step
+ *  errstr           -- set to a more text error message from
+ *                    a lower level mechanism on failure
+ *
+ * returns:
+ *  SASL_OK        -- exchange is complete.
+ *  SASL_CONTINUE  -- indicates another step is necessary.
+ *  SASL_TRANS     -- entry for user exists, but not for mechanism
+ *                    and transition is possible
+ *  SASL_BADPARAM  -- service name needed
+ *  SASL_BADPROT   -- invalid input from client
+ *  ...
+ */
+
 int sasl_server_step(sasl_conn_t *conn,
 		     const char *clientin,
 		     unsigned clientinlen,
@@ -646,9 +855,14 @@ int sasl_server_step(sasl_conn_t *conn,
 		     unsigned *serveroutlen,
 		     const char **errstr)
 {
+  /* cast */
   sasl_server_conn_t *s_conn;
   s_conn= (sasl_server_conn_t *) conn;
 
+  /* check parameters */
+  if ((clientin==NULL) && (clientinlen>0))
+    return SASL_BADPARAM;
+
   if (errstr)
     *errstr = NULL;
 
@@ -662,6 +876,9 @@ int sasl_server_step(sasl_conn_t *conn,
 				       errstr);
 }
 
+/* returns the length of all the mechanisms
+ * added up 
+ */
 
 static unsigned mech_names_len()
 {
@@ -676,6 +893,27 @@ static unsigned mech_names_len()
   return result;
 }
 
+/* This returns a list of mechanisms in a NUL-terminated string
+ *  user          -- restricts mechanisms to those available to that user
+ *                   (may be NULL)
+ *  prefix        -- appended to beginning of result
+ *  sep           -- appended between mechanisms
+ *  suffix        -- appended to end of result
+ * results:
+ *  result        -- NUL terminated allocated result, caller must free
+ *  plen          -- gets length of result (excluding NUL), may be NULL
+ *  pcount        -- gets number of mechanisms, may be NULL
+ *
+ * returns:
+ *  SASL_OK        -- success
+ *  SASL_NOMEM     -- not enough memory
+ *  SASL_NOMECH    -- no enabled mechanisms
+ */
+
+/*
+ * The default behavior is to seperate with spaces if sep==NULL
+ */
+
 int sasl_listmech(sasl_conn_t *conn,
 		  const char *user __attribute__((unused)),
 		  const char *prefix,
@@ -725,18 +963,21 @@ int sasl_listmech(sasl_conn_t *conn,
     /* XXX This should be done with a callback function */
     if (mech_permitted(conn, listptr->plug))
     {
-      strcat(*result,listptr->plug->mech_name);
       if (pcount!=NULL)
 	(*pcount)++;
 
-      if (listptr->next!=NULL)
+      /* print seperator */      
+      if (lup>0)
       {
 	if (sep)
 	  strcat(*result,sep);
 	else
-	  strcat(*result," ");
+	  strcat(*result," "); /* if seperator is NULL give it space */
       }
 
+      /* now print the mechanism name */
+      strcat(*result,listptr->plug->mech_name);
+
     }
 
     listptr=listptr->next;
@@ -752,3 +993,83 @@ int sasl_listmech(sasl_conn_t *conn,
   
 }
 
+/* returns OK if it's valid */
+static int _sasl_checkpass(const char *mech, const char *service,
+			   const char *user, const char *pass)
+{
+    int result = SASL_NOMECH;
+
+#ifdef HAVE_PAM
+    if (!strcmp(mech, "PAM")) {
+	result = _sasl_PAM_verify_password(user, pass, NULL);
+    }
+#endif
+
+    if (!strcmp(mech, "passwd")) {
+	result = _sasl_passwd_verify_password(user, pass, NULL);
+    }
+
+    if (!strcmp(mech, "shadow")) {
+	result = _sasl_shadow_verify_password(user, pass, NULL);
+    }
+
+#ifdef HAVE_KRB
+    if (!strcmp(mech, "kerberos_v4")) {
+	/* check against krb */
+	result = _sasl_kerberos_verify_password(user, pass, service, NULL);
+    }
+#endif
+
+    return result;
+}
+
+
+/* check if a plaintext password is valid
+ * if user is NULL, check if plaintext is enabled
+ * inputs:
+ *  user         -- user to query in current user_realm
+ *  userlen      -- length of username, 0 = strlen(user)
+ *  pass         -- plaintext password to check
+ *  passlen      -- length of password, 0 = strlen(pass)
+ * outputs:
+ *  errstr       -- set to error message for use in protocols
+ * returns 
+ *  SASL_OK      -- success
+ *  SASL_NOMECH  -- user found, but no verifier
+ *  SASL_NOUSER  -- user not found
+ */
+int sasl_checkpass(sasl_conn_t *conn,
+		   const char *user,
+		   unsigned userlen __attribute__((unused)),
+		   const char *pass,
+		   unsigned passlen,
+		   const char **errstr)
+{
+    const char *mech = NULL;
+    int result = SASL_NOMECH;
+    sasl_getopt_t *getopt;
+    void *context;
+
+    if (user == NULL) return SASL_NOUSER;
+
+    if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context) 
+	    == SASL_OK) {
+	getopt(context, NULL, "pwcheck_method", &mech, NULL);
+    }
+
+    if (mech == NULL) {
+	mech = DEFAULT_PLAIN_MECHANISM;
+    }
+
+    result = _sasl_checkpass(mech, conn->service, user, pass);
+    *errstr = NULL;
+
+    if (result == SASL_OK) {
+	result = _sasl_strdup(user, &(conn->oparams.authid), NULL);
+	if (result != SASL_OK) return result;
+      
+	_sasl_transition(conn, pass, passlen);
+    }
+
+    return result;
+}
