@@ -24,6 +24,9 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 ******************************************************************/
 
+#define IM_BROKEN		/* integrity/encryption are very broken
+				   in this implementation */
+
 #include <config.h>
 #include <des.h>
 #include <stdlib.h>
@@ -39,20 +42,14 @@ SOFTWARE.
 
 #ifdef WIN32
 # include <winsock.h>
-#else
-//# include <sys/param.h>
-//# include <sys/socket.h>
+#else /* Unix */
 # include <netinet/in.h>
-//# include <arpa/inet.h>
-//# include <netdb.h>
 #endif /* WIN32 */
 
 #include <sasl.h>
 #include <saslplug.h>
-#include <saslutil.h>
 
-/* defines */
-/* #define SEND_REATH_RESPONSE */
+/* Definitions */
 
 #define NONCE_SIZE (32)		/* arbitrary */
 #define DIGEST_NOLAYER    (1)
@@ -62,14 +59,16 @@ SOFTWARE.
 #ifdef WIN32
 /* This must be after sasl.h, saslutil.h */
 #include "saslDIGESTMD5.h"
-#endif				/* WIN32 */
+#else /* Unix */
+extern int      strcasecmp(const char *s1, const char *s2);
+#endif /* end WIN32 */
 
 static const char rcsid[] = "$Implementation: Carnegie Mellon SASL "
 VERSION " $";
 
 #ifdef L_DEFAULT_GUARD
 #undef L_DEFAULT_GUARD
-#define L_DEFAULT_GUARD (0)
+#define L_DEFAULT_GUARD (1)
 #endif
 
 /* external definitions */
@@ -78,7 +77,6 @@ VERSION " $";
 /* gotta define gethostname ourselves on suns */
 extern int      gethostname(char *, int);
 #endif
-int             strcasecmp(const char *s1, const char *s2);
 
 
 #define bool int
@@ -101,20 +99,35 @@ typedef unsigned char HASHHEX[HASHHEXLEN + 1];
 #define CIPHER_RC440 16
 #define CIPHER_RC456 32  /* xxx this still here? */
 
+#ifdef DIGEST_DRAFT_2
+#define MAC_SIZE 8
+#else
+#define MAC_SIZE 10
+#endif
+
+#define SEALING_CLIENT_SERVER "Digest H(A1) to client-to-server sealing key magic constant"
+#define SEALING_SERVER_CLIENT "Digest H(A1) to server-to-client sealing key magic constant"
+
+#define SIGNING_CLIENT_SERVER "Digest session key to client-to-server signing key magic constant"
+#define SIGNING_SERVER_CLIENT "Digest session key to server-to-client signing key magic constant"
+
+#define SERVER 0
+#define CLIENT 1
 
 /* function definitions for cipher encode/decode */
 typedef int cipher_function_t(void *,
-			     const char *,
-			     unsigned,
-			     char **,
-			     unsigned *);
+			      const char *,
+			      unsigned,
+			      char **,
+			      unsigned *);
 
-typedef int cipher_init_t(void *,
+typedef int cipher_init_t(void *, sasl_utils_t *,
 			  char *, int);
 
 /* context that stores info */
 typedef struct context {
   int             state;	/* state in the authentication we are in */
+  int i_am;			/* are we the client or server? */
 
   unsigned char  *nonce;
   int             noncelen;
@@ -134,8 +147,8 @@ typedef struct context {
 
   /* function pointers */
   void            (*hmac_md5) (const unsigned char *text, int text_len,
-		                      const unsigned char *key, int key_len,
-			                       unsigned char[16]);
+			       const unsigned char *key, int key_len,
+			       unsigned char[16]);
   sasl_malloc_t  *malloc;
   sasl_free_t    *free;
 
@@ -146,8 +159,6 @@ typedef struct context {
   int             size;
   int             needsize;
 
-
-
   /* Server MaxBuf for Client or Client MaxBuf For Server */
   unsigned int    maxbuf;
 
@@ -166,30 +177,18 @@ typedef struct context {
   des_key_schedule keysched_dec2;   /* key schedule for 3des initialization */
 
 #ifdef WITH_RC4
-  rc4_context_t rc4_context;
+  rc4_context_t *rc4_enc_context;
+  rc4_context_t *rc4_dec_context;
 #endif /* WITH_RC4 */
-}               context_t;
+} context_t;
 
+/* this is from the rpc world */
 #define IN
 #define OUT
 
-/* Forward declarations: */
-/*
- * static char * calculate_response (context_t * text, sasl_utils_t * utils,
- * unsigned char *username, unsigned char *realm, unsigned char *nonce,
- * unsigned char *ncvalue, unsigned char *cnonce, char *qop, unsigned char
- * *digesturi, sasl_secret_t *passwd, bool     IsUTF8, OUT char
- * **response_value);
- */
 
 static int      htoi(unsigned char *hexin, int *res);
 
-#define MAGIC_IC "Digest session key to client-to-server signing key magic constant"
-#define MAGIC_IS "Digest session key to client-to-server signing key magic constant"
-
-#define MAGIC_CC "Digest H(A1) to client-to-server sealing key magic constant"
-#define MAGIC_CS "Digest H(A1) to client-to-server sealing key magic constant"
-
 #define DIGESTMD5_VERSION (3)
 #define KEYS_FILE NULL
 
@@ -337,6 +336,8 @@ DigestCalcHA1(IN context_t * text,
 						 * create Kic or Kis */
 	      IN unsigned char *pszMagic_c,	/* Magic constant used to
 						 * create Kcc or Kcs */
+	      IN unsigned int  n,         /* number of bits to use for the
+					     privacy key */
 	      OUT HASHHEX SessionKey)
 {
   MD5_CTX         Md5Ctx;
@@ -367,12 +368,6 @@ DigestCalcHA1(IN context_t * text,
   utils->MD5Update(&Md5Ctx, HA1, HASHLEN);
   utils->MD5Update(&Md5Ctx, pszMagic_i, strlen((char *) pszMagic_i));
   utils->MD5Final(text->Ki, &Md5Ctx);
-
-  /* xxx rc-* use different n */
-  utils->MD5Init(&Md5Ctx);
-  utils->MD5Update(&Md5Ctx, HA1, HASHLEN);
-  utils->MD5Update(&Md5Ctx, pszMagic_c, strlen((char *) pszMagic_c));
-  utils->MD5Final(text->Kc, &Md5Ctx);
 }
 
 
@@ -452,6 +447,7 @@ calculate_response(context_t * text,
 						 * create Kic or Kis */
 		   IN unsigned char *magic_c,	/* Magic constant used to
 						 * create Kcc or Kcs */
+		   IN unsigned int n, /* number of bits to use for privacy key */
 		   OUT char **response_value)
 {
   HASHHEX         SessionKey;
@@ -484,6 +480,7 @@ calculate_response(context_t * text,
 		cnonce,
 		magic_i,
 		magic_c,
+		n,
 		SessionKey);
 
   VL(("Session Key is \"%s\"\r\n", SessionKey));
@@ -546,6 +543,7 @@ DigestCalcHA1FromSecret(IN context_t * text,
 							 * to create Kic or Kis */
 			IN unsigned char *pszMagic_c,	/* Magic constant used
 							 * to create Kic or Kis */
+			IN unsigned int n, /* number of bits to use for privacy key */
 			OUT HASHHEX SessionKey)
 {
   MD5_CTX         Md5Ctx;
@@ -569,22 +567,11 @@ DigestCalcHA1FromSecret(IN context_t * text,
   utils->MD5Update(&Md5Ctx, HA1, HASHLEN);
   utils->MD5Update(&Md5Ctx, pszMagic_i, strlen((char *) pszMagic_i));
   utils->MD5Final(text->Ki, &Md5Ctx);
-
-  /* for confedentiality protection Kcs = MD5(H(A1),"session key") */
-  utils->MD5Init(&Md5Ctx);
-  utils->MD5Update(&Md5Ctx, HA1, HASHLEN); /* 16 here */
-  utils->MD5Update(&Md5Ctx, pszMagic_c, strlen((char *) pszMagic_c));
-  utils->MD5Final(text->Kc, &Md5Ctx);  
 }
 
 static char    *
 create_response(context_t * text,
 		sasl_utils_t * utils,
-/* can we get rid of username? */
-/*
- * char *username __attribute__((unused)), char *realm
- * __attribute__((unused)),
- */
 		unsigned char *nonce,
 		unsigned char *ncvalue,
 		unsigned char *cnonce,
@@ -595,6 +582,7 @@ create_response(context_t * text,
 						 * create Kic or Kis */
 		IN unsigned char *magicc,	/* Magic constant used to
 						 * create Kcc or Kcs */
+		IN unsigned int n,  /* number of bits used for privacy key */
 		OUT char **response_value)
 {
   HASHHEX         SessionKey;
@@ -612,6 +600,7 @@ create_response(context_t * text,
 			  cnonce,
 			  magici,
 			  magicc,
+			  n,
 			  SessionKey);
 
 
@@ -798,10 +787,14 @@ get_pair(char **in, char **name, char **value)
   *name = NULL;
   *value = NULL;
 
-  if (curp == NULL)
-    return;
-  if (curp[0] == '\0')
-    return;
+  if (curp == NULL) {
+      *name = NULL;
+      return;
+  }
+  if (curp[0] == '\0') {
+      *name = NULL;
+      return;
+  }
 
   /* skipping spaces: */
   while (curp[0] == ' ')
@@ -810,6 +803,10 @@ get_pair(char **in, char **name, char **value)
   *name = curp;
 
   *value = strchr(*name, '=');
+  if (*value == NULL) {
+      *name = NULL;
+      return;
+  }
   (*value)[0] = '\0';
   (*value)++;
 
@@ -952,19 +949,48 @@ int enc_3des(struct context *text,
 
   return SASL_OK;
 }
-static int init_3des(context_t *text, char *key, int keylen)
+
+static int init_3des(context_t *text, sasl_utils_t *utils, 
+		     char *key, int keylen)
 {
-  if (keylen<14) return SASL_FAIL;
+    char enckey[16];
+    char deckey[16];
+    MD5_CTX Md5Ctx;
+    
+    if (text == NULL) return SASL_BADPARAM;
+    if (key == NULL) return SASL_BADPARAM;
+    
+    VL(("initializing 3des\n"));
+  
+    utils->MD5Init(&Md5Ctx);
+    utils->MD5Update(&Md5Ctx, key, keylen);
+    if (text->i_am == SERVER) {
+	utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT, 
+			 strlen(SEALING_SERVER_CLIENT));
+    } else {
+	utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER,
+			 strlen(SEALING_SERVER_CLIENT));
+    }
+    utils->MD5Final(enckey, &Md5Ctx);
+    
+    utils->MD5Init(&Md5Ctx);
+    utils->MD5Update(&Md5Ctx, key, keylen);
+    if (text->i_am == SERVER) {
+	utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER, 
+			 strlen(SEALING_CLIENT_SERVER));
+    } else {
+	utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT,
+			 strlen(SEALING_CLIENT_SERVER));
+    }
+    utils->MD5Final(deckey, &Md5Ctx);
+    
+    des_key_sched((des_cblock *) enckey, text->keysched_enc);
+    des_key_sched((des_cblock *) deckey, text->keysched_dec);
+    
+    des_key_sched((des_cblock *) (enckey+7), text->keysched_enc2);
+    des_key_sched((des_cblock *) (deckey+7), text->keysched_dec2);
 
-  VL(("initializing 3des\n"));
-
-  des_key_sched(key, text->keysched_enc); /* encryption and decryption */
-  des_key_sched(key, text->keysched_dec); /* encryption and decryption */
-
-  des_key_sched(key+7, text->keysched_enc2); /* encryption and decryption */
-  des_key_sched(key+7, text->keysched_dec2); /* encryption and decryption */
-
-  return SASL_OK;
+    return SASL_OK;
 }
 
 
@@ -1002,8 +1028,7 @@ static int dec_des(context_t *text,
 		    DES_DECRYPT);
   }
 
-  while( (*output)[(*outputlen)-1]==0)
-    (*outputlen)--;
+  /* HOW CAN WE STRIP OFF THE PADDING?!?!? */
 
   return SASL_OK;
 }
@@ -1027,7 +1052,9 @@ static int enc_des(context_t *text,
   {
     if (inputlen-lup<8)
     {
-      memset(last,0,8);
+      memset(last,8-(inputlen-lup),8); /* "padding prefix is one or more 
+					  octets each containing 
+					  the number of padding bytes" */
       memcpy(last,input+lup,inputlen-lup);
       /* encrpyt with 1st key */
       des_ecb_encrypt((des_cblock *) last,
@@ -1048,25 +1075,88 @@ static int enc_des(context_t *text,
 
   return SASL_OK;
 }
-static int init_des(context_t *text, char *key, int keylen)
+
+static int init_des(context_t *text, sasl_utils_t *utils,
+		    char *key, int keylen)
 {
-  if (keylen<7) return SASL_FAIL;
-
-  VL(("initializing des\n"));
-
-  des_key_sched(key, text->keysched_enc); /* encryption and decryption */
-  des_key_sched(key, text->keysched_dec); /* encryption and decryption */
-
-  return SASL_OK;
+    char enckey[16];
+    char deckey[16];
+    MD5_CTX Md5Ctx;
+    
+    if (text == NULL) return SASL_BADPARAM;
+    if (key == NULL) return SASL_BADPARAM;
+    
+    utils->MD5Init(&Md5Ctx);
+    utils->MD5Update(&Md5Ctx, key, keylen);
+    if (text->i_am == SERVER) {
+	utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT, 
+			 strlen(SEALING_SERVER_CLIENT));
+    } else {
+	utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER,
+			 strlen(SEALING_SERVER_CLIENT));
+    }
+    utils->MD5Final(enckey, &Md5Ctx);
+    
+    utils->MD5Init(&Md5Ctx);
+    utils->MD5Update(&Md5Ctx, key, keylen);
+    if (text->i_am == SERVER) {
+	utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER, 
+			 strlen(SEALING_CLIENT_SERVER));
+    } else {
+	utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT,
+			 strlen(SEALING_CLIENT_SERVER));
+    }
+    utils->MD5Final(deckey, &Md5Ctx);
+    
+    VL(("initializing DES\n"));
+    
+    des_key_sched((des_cblock *) enckey, text->keysched_enc);
+    des_key_sched((des_cblock *) deckey, text->keysched_dec);
+    
+    return SASL_OK;
 }
 
 #ifdef WITH_RC4
 static int
-init_rc4(context_t *text,
-	 char *key,
-	 int keylen)
+init_rc4(void *v, sasl_utils_t *utils,
+	 char *key, int keylen)
 {
-  rc4_init(&text->rc4_context, key, keylen);
+  context_t *text = (context_t *) v;
+  char enckey[16];
+  char deckey[16];
+  MD5_CTX Md5Ctx;
+
+  utils->MD5Init(&Md5Ctx);
+  utils->MD5Update(&Md5Ctx, key, keylen);
+  if (text->i_am == SERVER) {
+      utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT, 
+		       strlen(SEALING_SERVER_CLIENT));
+  } else {
+      utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER,
+		       strlen(SEALING_SERVER_CLIENT));
+  }
+  utils->MD5Final(enckey, &Md5Ctx);
+
+  utils->MD5Init(&Md5Ctx);
+  utils->MD5Update(&Md5Ctx, key, keylen);
+  if (text->i_am == SERVER) {
+      utils->MD5Update(&Md5Ctx, SEALING_CLIENT_SERVER, 
+		       strlen(SEALING_CLIENT_SERVER));
+  } else {
+      utils->MD5Update(&Md5Ctx, SEALING_SERVER_CLIENT,
+		       strlen(SEALING_CLIENT_SERVER));
+  }
+  utils->MD5Final(deckey, &Md5Ctx);
+
+  text->rc4_enc_context=(rc4_context_t *) text->malloc(sizeof(rc4_context_t));
+  if (text->rc4_enc_context==NULL) return SASL_NOMEM;
+
+  text->rc4_dec_context=(rc4_context_t *) text->malloc(sizeof(rc4_context_t));
+  if (text->rc4_dec_context==NULL) return SASL_NOMEM;
+
+  rc4_init(text->rc4_enc_context, enckey, 16);
+  rc4_init(text->rc4_dec_context, deckey, 16);
+
   return SASL_OK;
 }
 
@@ -1080,7 +1170,7 @@ dec_rc4(context_t *text,
   *output = (char *) text->malloc(inputlen);
   if (*output == NULL) return SASL_NOMEM;
   *outputlen = inputlen;
-  rc4_decrypt(&text->rc4_context, input, *output, inputlen);
+  rc4_decrypt(text->rc4_dec_context, input, *output, inputlen);
   return SASL_OK;
 }
 
@@ -1092,10 +1182,38 @@ enc_rc4(context_t *text,
 	unsigned *outputlen)
 {
   *outputlen = inputlen;
-  rc4_encrypt(&text->rc4_context, input, *output, inputlen);
+  rc4_encrypt(text->rc4_enc_context, input, *output, inputlen);
   return SASL_OK;
 }
 
+#else
+
+static int
+init_rc4(void *v, sasl_utils_t *utils, char *key, int keylen)
+{
+    return SASL_FAIL;
+}
+
+static int
+dec_rc4(context_t *text,
+	const char *input,
+	unsigned inputlen,
+	char **output,
+	unsigned *outputlen)
+{
+    return SASL_FAIL;
+}
+
+static int
+enc_rc4(context_t *text,
+	const char *input,
+	unsigned inputlen,
+	char **output,
+	unsigned *outputlen)
+{
+    return SASL_FAIL;
+}
+
 #endif /* WITH_RC4 */
 
 static unsigned int version = 1;
@@ -1137,6 +1255,7 @@ privacy_encode(void *context,
   /* construct (seqnum, msg) */
   param2 = (unsigned char *) text->malloc(inputlen + 4);
   if (param2 == NULL) return SASL_NOMEM;
+
   tmpnum = htonl(text->seqnum);
   memcpy(param2, &tmpnum, 4);
   memcpy(param2 + 4, input, inputlen);
@@ -1146,11 +1265,13 @@ privacy_encode(void *context,
   text->hmac_md5(text->Ki, HASHLEN,
 		 param2, inputlen + 4, digest);
 
+  text->free(param2);
+
   /* MAC foo */
-  text->cipher_enc(text, digest, 8,
+  text->cipher_enc(text, digest, MAC_SIZE,
 		   &out, &tmpnum);
 
-  out+=8;
+  out+=MAC_SIZE;
 
   /* put in seqnum */
   tmpnum = htonl(text->seqnum);
@@ -1187,6 +1308,7 @@ privacy_decode(void *context,
     unsigned char *macmid;
     int tmpnum;
     int lup;
+    unsigned char *tmpbuf;
 
     if (text->needsize>0) /* 4 bytes for how long message is */
     {
@@ -1211,7 +1333,11 @@ privacy_decode(void *context,
 	text->cursize=0;
 	text->size=ntohl(text->size);
 
-	if (text->size>0xFFFF) return SASL_FAIL; /* too big probably error */
+	/* No, this is not an error! Maximal size used in GSSAPI K5 is
+	   0xFFFFFF, but not 0xFFFF 
+	   -this is according to john myers at least
+	*/
+	if (text->size>0xFFFFFF) return SASL_FAIL; /* too big probably error */
 	
 	text->buffer=text->malloc(text->size+5);
 	if (text->buffer == NULL) return SASL_NOMEM;
@@ -1254,6 +1380,8 @@ privacy_decode(void *context,
 
     /* check the CMAC */
 
+    /* xxx check version number */
+
     /* construct (seqnum, msg) */
     param2 = (unsigned char *) text->malloc((*outputlen) + 4);
     if (param2 == NULL) return SASL_NOMEM;
@@ -1265,19 +1393,44 @@ privacy_decode(void *context,
     text->hmac_md5(text->Ki, HASHLEN,
 		   param2, (*outputlen) + 4, digest);
 
-    macmid=(char *)malloc(10);
+    text->free(param2);
+
+
+
 
     /* MAC foo */
-    text->cipher_dec(text, (text->buffer)+text->size-12, 8,
-		     (char **) &macmid, &tmpnum);
 
-		     
-    for (lup=0;lup<8;lup++)
-      if (macmid[lup]!=digest[lup])
-      {
-	VL(("CMAC doesn't match!\n"));
-	return SASL_FAIL;
-      }
+    /* this sucks. we want to encode but we want to use rc4's decode sbox's 
+     * so stuff doesn't get out of sync 
+     */
+#if HAVE_RC4
+    if (text->cipher_init==(&init_rc4))
+#else
+    if (0)
+#endif
+    {
+      text->cipher_dec(text, digest, MAC_SIZE,
+		       (char **) &macmid, &tmpnum);
+    } else { /* else is DES */
+      macmid=(char *)malloc(MAC_SIZE+12);
+      text->cipher_enc(text, digest, MAC_SIZE,
+		       (char **) &macmid, &tmpnum);
+    }
+  /*    text->cipher_dec(text, (text->buffer)+text->size-(MAC_SIZE+4), MAC_SIZE,
+	(char **) &macmid, &tmpnum);*/
+
+      /*      printf("%i. %i %i\n",lup,macmid[lup],((unsigned char *)(text->buffer)+text->size-(MAC_SIZE+4))[lup]);*/
+
+  tmpbuf=((unsigned char *)(text->buffer)+text->size-(MAC_SIZE+4));
+
+  for (lup=0;lup<MAC_SIZE;lup++) /*     printf("%i %i %i\n",lup,macmid[lup],tmpbuf[lup]);
+				  */
+    if (macmid[lup]!=tmpbuf[lup])
+    {
+      text->free(macmid);
+      VL(("CMAC doesn't match!\n"));
+      return SASL_FAIL;
+    }
     
     text->free(macmid);
     text->free(text->buffer);
@@ -1289,7 +1442,7 @@ privacy_decode(void *context,
     /* if received more than the end of a packet */
     if (inputlen!=0)
     {
-      
+      extra=NULL;
       privacy_decode(text, input, inputlen,
 			   &extra, &extralen);
       if (extra!=NULL) /* if received 2 packets merge them together */
@@ -1337,8 +1490,13 @@ integrity_encode(void *context,
 
   /* create MAC */
   tmpnum = htonl(version);
+#ifdef DIGEST_DRAFT_2
   memcpy(MAC, &tmpnum, 4);	/* 4 bytes = version */
-  memcpy(MAC + 4, digest, 8);	/* 8 bytes = first 8 bytes of the hmac */
+#else
+  memcpy(MAC, &tmpnum, 2);	/* 2 bytes = version */
+#endif
+  /* xxx i think this is wrong */
+  memcpy(MAC + 4, digest, MAC_SIZE);	/* n bytes = first n bytes of the hmac */
   tmpnum = htonl(text->seqnum);
   memcpy(MAC + 12, &tmpnum, 4);	/* 4 bytes = sequence number */
 
@@ -1398,8 +1556,14 @@ create_MAC(context_t * text,
 
   /* create MAC */
   tmpnum = htonl(version);
+#ifdef DIGEST_DRAFT_2
   memcpy(MAC, &tmpnum, 4);	/* 4 bytes = version */
   memcpy(MAC + 4, digest, 8);	/* 8 bytes = first 8 bytes of the hmac */
+#else
+  memcpy(MAC, &tmpnum, 2);	/* 2 bytes = version */
+  memcpy(MAC + 2, digest, 10);	/* 10 bytes = first 10 bytes of the hmac */
+#endif
+
   tmpnum = htonl(seqnum);
   memcpy(MAC + 12, &tmpnum, 4);	/* 4 bytes = sequence number */
 
@@ -1534,28 +1698,29 @@ integrity_decode(void *context,
 }
 
 
-static int      server_start(void *glob_context __attribute__((unused)),
-			                     sasl_server_params_t * sparams,
-	                      const char *challenge __attribute__((unused)),
-			                int challen __attribute__((unused)),
-			                     void **conn,
-			                     const char **errstr) {
-  context_t      *text;
+static int server_start(void *glob_context __attribute__((unused)),
+			sasl_server_params_t * sparams,
+			const char *challenge __attribute__((unused)),
+			int challen __attribute__((unused)),
+			void **conn,
+			const char **errstr)
+{
+    context_t *text;
 
-  if (errstr)
-    *errstr = NULL;
+    if (errstr)
+	*errstr = NULL;
 
-  /* holds state are in */
-  text = sparams->utils->malloc(sizeof(context_t));
-  if (text == NULL)
-    return SASL_NOMEM;
-  text->state = 1;
+    /* holds state are in */
+    text = sparams->utils->malloc(sizeof(context_t));
+    if (text == NULL)
+	return SASL_NOMEM;
 
-  text->cipher_init=NULL;
+    text->i_am = SERVER;
+    text->state = 1;
+    text->cipher_init=NULL;
+    *conn = text;
 
-  *conn = text;
-
-  return SASL_OK;
+    return SASL_OK;
 }
 
 static void
@@ -1576,11 +1741,17 @@ static int
 get_realm(sasl_server_params_t * params,
 	  char **realm)
 {
-  if (params->local_domain == NULL) {
+  /* look at user realm first */
+  if (params->user_realm != NULL)
+  {
+    *realm = (char *) params->user_realm;  
+
+  } else if (params->serverFQDN != NULL) {
+    *realm = (char *) params->serverFQDN;
+
+  } else {
     VL(("No way to obtain domain\n"));
     return SASL_FAIL;
-  } else {
-    *realm = (char *) params->local_domain;
   }
 
   return SASL_OK;
@@ -1609,7 +1780,7 @@ server_continue_step(void *conn_context,
     unsigned char  *nonce;
     char           *qop = "auth,auth-int,auth-conf";
 #ifdef WITH_RC4
-    char           *cipheropts="3des,des,rc4";
+    char           *cipheropts="3des,des,rc4,rc4-40,rc4-56";
 #else
     char           *cipheropts="3des,des";
 #endif
@@ -1621,6 +1792,11 @@ server_continue_step(void *conn_context,
      * charset | cipher-opts | auth-param )
      */
 
+#ifdef IM_BROKEN
+    cipheropts = "frog";
+    qop = "auth";
+#endif
+
     /* get realm */
     result = get_realm(sparams, &realm);
 
@@ -1739,11 +1915,12 @@ server_continue_step(void *conn_context,
 
     char           *maxbufstr = NULL;
 
-    unsigned int    client_maxbuf = 65536;	/* xxx is this right??? */
+    unsigned int    client_maxbuf = 2096;	/* xxx is this right??? */
     int             maxbuf_count = 0;	/* How many maxbuf instaces was found */
 
     char           *charset = NULL;
     char           *cipher = NULL;
+    unsigned int   n=0;
 
     HASH            A1;
 
@@ -1754,9 +1931,7 @@ server_continue_step(void *conn_context,
     char           *in_start;
     char           *in = sparams->utils->malloc(clientinlen + 1);
 
-#ifdef SEND_REATH_RESPONSE
     char *response_auth = NULL;
-#endif
 
     memcpy(in, clientin, clientinlen);
     in[clientinlen] = 0;
@@ -1766,9 +1941,12 @@ server_continue_step(void *conn_context,
 
     /* parse what we got */
     while (in[0] != '\0') {
-      char           *name, *value;
+      char           *name = NULL, *value = NULL;
       get_pair(&in, &name, &value);
 
+      if (name == NULL)
+	  break;
+
       VL(("received form client pair: %s - %s\n", name, value));
 
       /* Extracting parameters */
@@ -1870,6 +2048,14 @@ server_continue_step(void *conn_context,
 
     }
 
+
+    /* defaulting qop to "auth" if not specified */
+
+    if (qop == NULL)
+      digest_strdup(sparams->utils, "auth", &qop, NULL);      
+
+
+
     /* check which layer/cipher to use */
 
     if (strcmp(qop, "auth-conf") == 0) {
@@ -1885,18 +2071,38 @@ server_continue_step(void *conn_context,
 	text->cipher_enc=(cipher_function_t *) &enc_des;
 	text->cipher_dec=(cipher_function_t *) &dec_des;
 	text->cipher_init=(cipher_init_t *) &init_des;	
-	oparams->mech_ssf = 56;
+	oparams->mech_ssf = 55;
+	n=16; /* number of bits to make privacy key */
+
       } else if (strcmp(cipher,"3des")==0) {
 	text->cipher_enc=(cipher_function_t *) &enc_3des;
 	text->cipher_dec=(cipher_function_t *) &dec_3des;
 	text->cipher_init=(cipher_init_t *) &init_3des;
 	oparams->mech_ssf = 112;
+	n=16;
+
 #ifdef WITH_RC4
       } else if (strcmp(cipher,"rc4")==0) {
 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
-	text->cipher_init=(cipher_init_t *) &init_rc4;
-	oparams->mech_ssf = 64;
+	text->cipher_init=&init_rc4;
+	oparams->mech_ssf = 128;
+ 	n = 16;
+ 
+      } else if (strcmp(cipher,"rc4-40")==0) {
+ 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
+ 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
+ 	text->cipher_init=&init_rc4;
+ 	oparams->mech_ssf = 40;
+ 	n = 5;
+
+      } else if (strcmp(cipher,"rc4-56")==0) {
+ 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
+ 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
+ 	text->cipher_init=&init_rc4;
+ 	oparams->mech_ssf = 56;
+ 	n = 7;
+ 
 #endif /* WITH_RC4 */
       } else {
 	VL(("Invalid or no cipher chosen\n"));
@@ -1954,9 +2160,11 @@ server_continue_step(void *conn_context,
      * alexey: I removed charset check, because I've misunderstood it purpose
      */
 
+    /* xxx not sure about this */
     if (qop == NULL)
       qop = "auth";
 
+
     usernamelen = strlen(username);
     realm_len = strlen(realm);
 
@@ -1967,7 +2175,7 @@ server_continue_step(void *conn_context,
       goto FreeAllMem;
     }
     memcpy(userid, username, usernamelen);
-    userid[usernamelen] = (char) ':';	/* '\0'; ??? */
+    userid[usernamelen] = (char) ':';
 
     memcpy(userid + usernamelen + 1, realm, realm_len);
     userid[usernamelen + realm_len + 1] = '\0';
@@ -2032,8 +2240,9 @@ server_continue_step(void *conn_context,
 				     qop,
 				     digesturi,
 				     A1,
-				     MAGIC_IS,
-				     MAGIC_CS,
+				     SIGNING_SERVER_CLIENT,
+				     SEALING_SERVER_CLIENT,
+				     n,
 				     &text->response_value);
 
 
@@ -2093,9 +2302,9 @@ server_continue_step(void *conn_context,
 
     /* initialize cipher if need be */
     if (text->cipher_init!=NULL)
-      text->cipher_init(text, text->Kc, 16);
-
-#ifdef SEND_REATH_RESPONSE
+      text->cipher_init(text, sparams->utils,
+			text->Kc, /* the privacy key */
+			n); /* number of bytes of key we made */
 
     /*
      * The server receives and validates the "digest-response". The server
@@ -2121,17 +2330,12 @@ server_continue_step(void *conn_context,
     *serverout = response_auth;
     *serveroutlen = strlen(response_auth);
 
+    /* self check */
     if (*serveroutlen > 2048) {
       result = SASL_FAIL;
       goto FreeAllMem;
     }
     result = SASL_CONTINUE;
-#else
-    *serverout = NULL;
-    *serveroutlen = 0;
-
-    result = SASL_OK;
-#endif
 
 FreeAllMem:
 
@@ -2154,17 +2358,12 @@ FreeAllMem:
 
     /* sparams->utils->free(serverresponse); */
 
-#ifdef SEND_REATH_RESPONSE
     if (result == SASL_CONTINUE)
       text->state = 3;
-#else
-    if (result == SASL_OK)
-      text->state = 99;		/* so it fails */
-#endif
 
     return result;
   }
-#ifdef SEND_REATH_RESPONSE
+
   if (text->state == 3) {
     VL(("Digest-MD5 Step 3\n"));
     /*
@@ -2181,19 +2380,19 @@ FreeAllMem:
 
     return SASL_OK;
   }
-#endif
+
 
   return SASL_FAIL;		/* should never get here */
 }
 
 static int
-                setpass(void *glob_context __attribute__((unused)),
-			                sasl_server_params_t * sparams,
-			                const char *user,
-			                const char *pass,
-			                unsigned passlen,
-			                int flags __attribute__((unused)),
-			                const char **errstr) {
+setpass(void *glob_context __attribute__((unused)),
+	sasl_server_params_t * sparams,
+	const char *user,
+	const char *pass,
+	unsigned passlen,
+	int flags __attribute__((unused)),
+	const char **errstr) {
   int             result;
   sasl_server_putsecret_t *putsecret;
   void           *putsecret_context;
@@ -2215,17 +2414,21 @@ static int
       || !pass)
     return SASL_BADPARAM;
 
-  if (!sparams->local_domain) {
+  /* get the realm */
+  result=get_realm(sparams,
+		   &realm);
+
+  if ((result!=SASL_OK) || (realm==NULL)) {
     VL(("Digest-MD5 requires a domain\n"));
     return SASL_NOTDONE;
   }
-  realm = (char *) sparams->local_domain;
+
   realmlen = strlen(realm);
   userlen = strlen(user);
 
   DigestCalcSecret(sparams->utils,
 		   (unsigned char *) user,
-		   (unsigned char *) sparams->local_domain,
+		   (unsigned char *) realm,
 		   (char *) pass,
 		   passlen,
 		   HA1);
@@ -2254,7 +2457,7 @@ static int
   memcpy(userid, user, userlen);
   userid[userlen] = (char) ':';
 
-  memcpy(userid + userlen + 1, sparams->local_domain, realmlen);
+  memcpy(userid + userlen + 1, realm, realmlen);
   userid[userlen + realmlen + 1] = '\0';
 
 
@@ -2277,8 +2480,16 @@ const sasl_server_plug_t plugins[] =
 {
   {
     "DIGEST-MD5",
-    0,				/* max ssf */
+#ifndef IM_BROKEN
+#ifdef WITH_RC4
+    128,				/*xxx max ssf */
+#else
+    112,
+#endif
+#else
     0,
+#endif
+    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
     NULL,
     &server_start,
     &server_continue_step,
@@ -2293,18 +2504,14 @@ const sasl_server_plug_t plugins[] =
   }
 };
 
-int             sasl_server_plug_init(sasl_utils_t * utils __attribute__((unused)),
-		                     int maxversion __attribute__((unused)),
-				                      int *out_version,
-		                       const sasl_server_plug_t ** pluglist,
-				                      int *plugcount) {
-  /* fail if we can't open the srvtab file */
-  /*
-   * if (access(KEYFILE, R_OK)!=0) //  return SASL_FAIL;
-   */
-
-  if (maxversion < DIGESTMD5_VERSION)
-    return SASL_BADVERS;
+int sasl_server_plug_init(sasl_utils_t * utils __attribute__((unused)),
+			  int maxversion __attribute__((unused)),
+			  int *out_version,
+			  const sasl_server_plug_t ** pluglist,
+			  int *plugcount) 
+{
+  /*  if (maxversion < DIGESTMD5_VERSION)
+      return SASL_BADVERS;*/
 
   *pluglist = plugins;
 
@@ -2315,25 +2522,25 @@ int             sasl_server_plug_init(sasl_utils_t * utils __attribute__((unused
 }
 
 /* put in sasl_wrongmech */
-static int      c_start(void *glob_context __attribute__((unused)),
-			                sasl_client_params_t * params,
-			                void **conn) {
-  context_t      *text;
+static int c_start(void *glob_context __attribute__((unused)),
+		   sasl_client_params_t * params,
+		   void **conn) {
+    context_t *text;
 
-  /* holds state are in */
-  text = params->utils->malloc(sizeof(context_t));
-  if (text == NULL)
-    return SASL_NOMEM;
+    /* holds state are in */
+    text = params->utils->malloc(sizeof(context_t));
+    if (text == NULL)
+	return SASL_NOMEM;
 
-  text->authid = NULL;
-  text->password = NULL;
+    text->i_am = CLIENT;
+    text->authid = NULL;
+    text->password = NULL;
+    text->cipher_init=NULL;
+    text->state = 1;
 
-  text->cipher_init=NULL;
+    *conn = text;
 
-  text->state = 1;
-  *conn = text;
-
-  return SASL_OK;
+    return SASL_OK;
 }
 
 
@@ -2633,42 +2840,28 @@ c_continue_step(void *conn_context,
   }
   if (text->state == 2) {
     unsigned char  *digesturi = NULL;
-
     unsigned char  *nonce = NULL;
     unsigned char  *ncvalue = (unsigned char *) "00000001";
     unsigned char  *cnonce = NULL;
-
     char           *qop = NULL;
     char           *qop_list = NULL;
     int             protection = 0;
-
     char           *cipher = NULL;
     char           *cipher_list = NULL;
     int             ciphers=0;
-
+    unsigned int  n=0;
     char           *response = NULL;
-
     char           *realm = NULL;
-
-    unsigned int    server_maxbuf = 65536;
-
+    unsigned int    server_maxbuf = 2096;
     int             maxbuf_count = 0;
-
     bool            IsUTF8 = FALSE;
-
     char           *charset = NULL;
-    /* char *cipher = NULL; unused */
-
     char           *xxx;
     char           *prev_xxx;
-
     int             result = SASL_FAIL;
-
     char           *client_response = NULL;
-
     sasl_security_properties_t secprops;
     int             external;
-
     int             auth_result = SASL_OK;
     int             pass_result = SASL_OK;
 
@@ -2813,9 +3006,18 @@ c_continue_step(void *conn_context,
 	    ciphers |= CIPHER_3DES;
 
 #ifdef WITH_RC4
-	  } else if (strcmp(prev_xxx, "rc4") == 0) {
-	    VL(("Server supports rc-4\n"));
-	    ciphers |= CIPHER_RC4;
+  	  } else if (strcmp(prev_xxx, "rc4") == 0) {
+ 	    VL(("Server supports rc4\n"));
+  	    ciphers |= CIPHER_RC4;
+ 
+ 	  } else if (strcmp(prev_xxx, "rc4-40") == 0) {
+ 	    VL(("Server supports rc4-40\n"));
+ 	    ciphers |= CIPHER_RC440;
+ 
+ 	  } else if (strcmp(prev_xxx, "rc4-56") == 0) {
+ 	    VL(("Server supports rc4-56\n"));
+ 	    ciphers |= CIPHER_RC456;
+ 
 #endif /* WITH_RC4 */
 	  } else {
 
@@ -2911,30 +3113,64 @@ c_continue_step(void *conn_context,
 
       /* Client request encryption, server support it */
       /* encryption */
-      if ((secprops.max_ssf>=112) && ((ciphers & CIPHER_3DES) == CIPHER_3DES))
-      {
+#ifdef WITH_RC4
+      if ((secprops.max_ssf>=128)  && 
+	  ((ciphers & CIPHER_RC4) == CIPHER_RC4)) { /* rc4 */
+#else
+      if (0) {
+#endif /* WITH_RC4 */
+	VL(("Trying to use rc4"));
+	cipher = "rc4";
+	text->cipher_enc=(cipher_function_t *) &enc_rc4; /* uses same function both ways */
+	text->cipher_dec=(cipher_function_t *) &dec_rc4;
+	text->cipher_init=&init_rc4;
+	oparams->mech_ssf = 128;
+	n=16;
+
+      } else if ((secprops.max_ssf>=112) && ((ciphers & CIPHER_3DES) == CIPHER_3DES)) {
 	VL(("Trying to use 3des"));
 	cipher = "3des";
 	text->cipher_enc=(cipher_function_t *) &enc_3des;
 	text->cipher_dec=(cipher_function_t *) &dec_3des;
 	text->cipher_init=(cipher_init_t *) &init_3des;
 	oparams->mech_ssf = 112; 
+	n=16; /* number of bits to use for privacy key */
+
+
+
 #ifdef WITH_RC4
-      } else if ((secprops.max_ssf>=64)  && ((ciphers & CIPHER_RC4) == CIPHER_RC4)) { /* rc4 */
-	VL(("Trying to use rc4"));
-	cipher = "rc4";
-	text->cipher_enc=(cipher_function_t *) &enc_rc4; /* uses same function both ways */
-	text->cipher_dec=(cipher_function_t *) &dec_rc4;
-	text->cipher_init=(cipher_init_t *) &init_rc4;
-	oparams->mech_ssf = 64;
+      } else if ((secprops.max_ssf>=56)  && ((ciphers & CIPHER_RC456) == CIPHER_RC456)) { /* rc4-56 */
+ 	VL(("Trying to use rc4-56"));
+ 	cipher = "rc4-56";
+ 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
+ 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
+ 	text->cipher_init=&init_rc4;
+ 	oparams->mech_ssf = 56;
+ 	n = 7;
 #endif /* WITH_RC4 */
-      } else if ((secprops.max_ssf>=56)  && ((ciphers & CIPHER_DES) == CIPHER_DES)) { /* des */
+
+
+      } else if ((secprops.max_ssf>=55)  && ((ciphers & CIPHER_DES) == CIPHER_DES)) { /* des */
 	VL(("Trying to use des"));
 	cipher = "des";
 	text->cipher_enc=(cipher_function_t *) &enc_des;
 	text->cipher_dec=(cipher_function_t *) &dec_des;
 	text->cipher_init=(cipher_init_t *) &init_des;
-	oparams->mech_ssf = 56; 
+	oparams->mech_ssf = 55; 
+	n=16;
+
+#ifdef WITH_RC4
+      } else if ((secprops.max_ssf>=40)  && ((ciphers & CIPHER_RC440) == CIPHER_RC440)) { /* rc4-40 */
+ 	VL(("Trying to use rc4-40"));
+ 	cipher = "rc4-40";
+ 	text->cipher_enc=(cipher_function_t *) &enc_rc4;
+ 	text->cipher_dec=(cipher_function_t *) &dec_rc4;
+ 	text->cipher_init=&init_rc4;
+ 	oparams->mech_ssf = 40;
+ 	n = 5;
+ 
+#endif /* WITH_RC4 */
+
       } else {
 	/* should try integrity or plain */
 	VL(("No good privacy layers\n"));
@@ -3017,8 +3253,9 @@ c_continue_step(void *conn_context,
 				  qop,
 				  digesturi,
 				  text->password,
-				  MAGIC_IC,
-				  MAGIC_CC,
+				  SIGNING_CLIENT_SERVER,
+				  SEALING_CLIENT_SERVER,
+				  n, /* bytes to use to make privacy key */
 				  &text->response_value);
 
     VL(("Constructing challenge\n"));
@@ -3078,7 +3315,8 @@ c_continue_step(void *conn_context,
       result = SASL_FAIL;
       goto FreeAllocatedMem;
     }
-    result = SASL_OK;
+
+    result = SASL_CONTINUE;
 
     text->state = 3;
 
@@ -3103,7 +3341,7 @@ c_continue_step(void *conn_context,
     /* set oparams */
 
     oparams->doneflag = 1;
-    oparams->maxoutbuf = 1024;	/* no clue what this should be */
+    oparams->maxoutbuf = 4096;
 
     oparams->param_version = 0;
 
@@ -3123,7 +3361,7 @@ c_continue_step(void *conn_context,
 
     /* initialize cipher if need be */
     if (text->cipher_init!=NULL)
-      text->cipher_init(text, text->Kc, 16);
+      text->cipher_init(text, params->utils, text->Kc, n);
 
 FreeAllocatedMem:
     params->utils->free(response);	/* !!! */
@@ -3157,7 +3395,7 @@ FreeAllocatedMem:
 
     return result;
   }
-#ifdef SEND_REATH_RESPONSE
+
   if (text->state == 3) {	/* Verify that server is really what he
      * claims to be *//* ReAUTH. NTI!!! */
 
@@ -3186,7 +3424,8 @@ FreeAllocatedMem:
 	  VL(("Ok I think we can re-auth\n"));
 	  params->utils->free(in_start);
 
-	  *clientout = NULL;
+ 	  *clientout = params->utils->malloc(1);
+	  (*clientout)[0] = '\0';
 	  *clientoutlen = 0;
 	  return SASL_OK;
 	}
@@ -3199,7 +3438,7 @@ FreeAllocatedMem:
 
     return SASL_FAIL;
   }
-#endif
+
 
   return SASL_FAIL;		/* should never get here */
 }
@@ -3215,8 +3454,16 @@ const sasl_client_plug_t client_plugins[] =
 {
   {
     "DIGEST-MD5",
-    1 /* ??? */ ,		/* max ssf */
+#ifndef IM_BROKEN
+#ifdef WITH_RC4
+    128,				/*xxx max ssf */
+#else
+    112,
+#endif
+#else
     0,
+#endif
+    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
     client_required_prompts,
     NULL,
     &c_start,
