@@ -73,8 +73,8 @@ static const char *flag_subopts[] = {
   "nodict",
 #define OPT_FORWARDSEC (3)
   "forwardsec",
-#define OPT_MAXIMUM (4)
-  "maximum",
+#define OPT_NOANONYMOUS (4)
+  "noanonymous",
 #define OPT_PASSCRED (5)
   "passcred",
   NULL
@@ -275,7 +275,7 @@ main(int argc, char *argv[])
   unsigned len, count;
   char *data;
   const char *errstr = NULL;
-  sasl_ssf_t ssf;
+  sasl_ssf_t *ssf;
 
   progname = strrchr(argv[0], '/');
   if (progname)
@@ -310,7 +310,7 @@ main(int argc, char *argv[])
 	default:
 	  errflag = 1;
 	  break;
-	}
+	  }
       break;
 
     case 'e':
@@ -332,7 +332,7 @@ main(int argc, char *argv[])
 	default:
 	  errflag = 1;
 	  break;
-	}
+	  } 
       break;
 
     case 'm':
@@ -355,8 +355,8 @@ main(int argc, char *argv[])
 	case OPT_FORWARDSEC:
 	  secprops.security_flags |= SASL_SEC_FORWARD_SECRECY;
 	  break;
-	case OPT_MAXIMUM:
-	  secprops.security_flags |= SASL_SEC_MAXIMUM;
+	case OPT_NOANONYMOUS:
+	  secprops.security_flags |= SASL_SEC_NOANONYMOUS;
 	  break;
 	case OPT_PASSCRED:
 	  secprops.security_flags |= SASL_SEC_PASS_CREDENTIALS;
@@ -364,9 +364,9 @@ main(int argc, char *argv[])
 	default:
 	  errflag = 1;
 	  break;
-	}
+	  }
 	if (value) errflag = 1;
-      }
+	}
       break;
 
     case 'i':
@@ -388,7 +388,7 @@ main(int argc, char *argv[])
 	default:
 	  errflag = 1;
 	  break;
-	}
+	  }
       break;
 
     case 'p':
@@ -480,8 +480,10 @@ main(int argc, char *argv[])
 
   if (mech) {
     printf("Forcing use of mechanism %s\n", mech);
-    data = mech;
-    len = strlen(mech);
+    data = strdup(mech);
+    if (! data)
+      osfail();
+    len = strlen(data);
     count = 1;
   } else {
     puts("Generating client mechanism list...");
@@ -557,7 +559,7 @@ main(int argc, char *argv[])
   if (result != SASL_OK)
     sasldebug(result, "ssf", NULL);
   else
-    printf("SSF: %d\n", ssf);
+    printf("SSF: %d\n", *ssf);
 
   return (EXIT_SUCCESS);
 }
